<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;

class Tco01006RegrasCardapioController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $entityManager;
    protected $newCode;
    protected $service;

    public function __construct($crudService, $parameterBag, $environment, $entityManager, $newCode, $service) {
        $this->crudService   = $crudService;
        $this->environment   = $environment;
        $this->entityManager = $entityManager;
        $this->newCode       = $newCode;
        $this->service       = $service;
    }
    
    //salva regra de adesão do cardápio base
    public function salvaRegraAdesao(DTO\Request $request, DTO\Response $response) {
        $regra = $request->getRow();
        $this->entityManager->beginTransaction();
        try{
            
            if($regra['__is_new']){
                $regra['NRADESAOCARDBASE'] = $this->newCode->getCodeByTableName('ADESAOCARDBASE', $this->environment->getNrOrgTrab());
                $regra['CDADESAOCARDBASE'] = $this->service->retornaProxCodAdesaoCard($regra['NRCARDAPIOBASE']);
            }
            $rows = $this->crudService->getRowsToSave($request, "adesaocardbase");
            $this->crudService->save("adesaocardbase", $rows);
            if($regra['__is_new']){
                $this->salvaSegmentoAdesao($regra);
                $this->salvaRamosAdesao($regra);
            }
            $this->salvaTiposDePrato($regra);
            
            $this->entityManager->commit();
            $response->addDataSet(new DataSet("chaves", array('NRADESAOCARDBASE' => $regra['NRADESAOCARDBASE'], 'CDADESAOCARDBASE'=> $regra['CDADESAOCARDBASE'])));
        }catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    //salva segmentos referentes à regra de adesão do cardápio base
    public function salvaSegmentoAdesao($regra){
        try{
            $dataSet = array();
            if(array_key_exists('CDSEGMER', $regra) && !empty($regra['CDSEGMER'])){
                foreach ($regra['CDSEGMER'] as $cdsegmer) {
                    $codigo = $this->newCode->getCodeByTableName('SEGMERADESAOCBASE', $this->environment->getNrOrgTrab());
                    $row = array(
                        'NRSEGMERADESAOCBASE'=> $codigo,
                        'NRADESAOCARDBASE'   => $regra['NRADESAOCARDBASE'],
                        'CDSEGMER'           => $cdsegmer,
                        '__is_new'           => true
                    );
                    array_push($dataSet, $row);
                }
            }
            $dataSet = new DataSet("toSave", $dataSet);
            $request = new DTO\Request\DataSet($dataSet, '', '', '');
            $rows    = $this->crudService->getRowsToSave($request, "segmeradesaocbase");
            $this->crudService->save("segmeradesaocbase", $rows);
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    //salva ramos de atividade referentes à regra de adesão do cardápio base
    public function salvaRamosAdesao($regra){
        try{
            $dataSet = array();
            if(array_key_exists('NRATIVCLISEGMER', $regra) && !empty($regra['NRATIVCLISEGMER'])){
                foreach ($regra['NRATIVCLISEGMER'] as $nrativclisegmer) {
                    $codigo = $this->newCode->getCodeByTableName('ATIVCLIADESAOCBASE', $this->environment->getNrOrgTrab());
                    $row = array(
                        'NRATIVCLIADESAOCBASE'=> $codigo,
                        'NRADESAOCARDBASE'    => $regra['NRADESAOCARDBASE'],
                        'NRATIVCLISEGMER'     => $nrativclisegmer,
                        '__is_new'            => true
                    );
                    array_push($dataSet, $row);
                }
            }
            $dataSet = new DataSet("toSave", $dataSet);
            $request = new DTO\Request\DataSet($dataSet, '', '', '');
            $rows    = $this->crudService->getRowsToSave($request, "ativcliadesaocbase");
            $this->crudService->save("ativcliadesaocbase", $rows);
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    //salva os tipos de prato referentes a uma regra de adesao do cardápio base
    public function salvaTiposDePrato($regra){
        $this->service->deletaTipoPratoRegraAdesao($regra['NRADESAOCARDBASE']);
        try{
            if(!empty($regra['TIPOSPRATO'])){
                
                foreach ($regra['TIPOSPRATO'] as $key => $value) {
                    $regra['TIPOSPRATO'][$key]['NRITEMADESAOCBASE'] = $this->newCode->getCodeByTableName('ITEMADESAOCBASE', $this->environment->getNrOrgTrab());
                    $regra['TIPOSPRATO'][$key]['NRADESAOCARDBASE'] = $regra['NRADESAOCARDBASE'];
                }
                
                $dataSet = new DataSet("toSave", $regra['TIPOSPRATO']);
                $request = new DTO\Request\DataSet($dataSet, '', '', '');
                $rows    = $this->crudService->getRowsToSave($request, "itemadesaocbase");
                $this->crudService->save("itemadesaocbase", $rows);
            }
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    //apaga regras de adesão do cardápio base
    public function deletaRegraAdesao(DTO\Request $request, DTO\Response $response) {
        $this->entityManager->beginTransaction();
        try{
            $rowsToDelete = $request->getDataSet()->getRows();
            foreach($rowsToDelete as $row){
                $nradesaocardbase = $row['NRADESAOCARDBASE'];
                $this->service->deletaTipoPratoRegraAdesao($nradesaocardbase);
                $this->service->deletaRegraAdesao($nradesaocardbase);
            }
            
            $rows = $this->crudService->getRowsToDelete($request, "adesaocardbase");
            $dataSet = new DataSet("adesaocardbase", $rows);
    		$this->crudService->delete($response, $dataSet);
    		
            $this->entityManager->commit();
        }catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    //retorna os valores referentes aos campos dinamicos para o grid de tipos de prato da regra de adesão
    public function retornaCamposDinamicosAdesao(DTO\Request $request, DTO\Response $response) {
        $filtro = $request->getRow();
        $conditions = [
            array(
                "columnName" => "NRORG",
                "operator"   => '=',
                "value"      => $this->environment->getNrOrgTrab()
            )
        ];
        $colunasDinamicas = $this->service->retornaColsPorcionamento($filtro['CDGRTIPOPR']);
        $colunasDinamicas = new DataSet('porcionamento', $colunasDinamicas);
        $response->addDataSet($colunasDinamicas);
        
        $conditions = [
            array(
                "columnName" => "CDGRTIPOPR",
                "operator"   => '=',
                "value"      => $filtro['CDGRTIPOPR']
            ),
            array(
                "columnName" => "NRADESAOCARDBASE",
                "operator"   => '=',
                "value"      => $filtro['NRADESAOCARDBASE']
            ),
            array(
                "columnName" => "NRCARDAPIOBASE",
                "operator"   => '=',
                "value"      => $filtro['NRCARDAPIOBASE']
            )
        ];
        $colunaNome = $this->crudService->getFilteredDataSet('tipoPratoAdesao', $conditions);
        $response->addDataSet($colunaNome);
        
        $conditions = [
            array(
                "columnName" => "NRADESAOCARDBASE",
                "operator"   => '=',
                "value"      => $filtro['NRADESAOCARDBASE']
            )
        ];
        $dados = $this->crudService->getFilteredDataSet('itemadesaocbase', $conditions);
        $response->addDataSet($dados);
    }
    
    //salva as regras de gramatura do cardápio base
    public function salvaRegraGramatura(DTO\Request $request, DTO\Response $response) {
        $regra = $request->getRow();
        $this->entityManager->beginTransaction();
        try{
            $unico = $regra['VERIFICA'] ? $this->notExistsGramatura($regra) : true;
            
            if($unico){
                if($regra['__is_new']){
                    $regra['NRGRAMATURACARDBASE'] = $this->newCode->getCodeByTableName('GRAMATURACARDBASE', $this->environment->getNrOrgTrab());
                }
                $rows = $this->crudService->getRowsToSave($request, "gramaturacardbase");
                $this->crudService->save("gramaturacardbase", $rows);
                
                $this->entityManager->commit();
                $response->addDataSet(new DataSet("chaves", array('NRGRAMATURACARDBASE' => $regra['NRGRAMATURACARDBASE'])));
            } else {
                $response->setError(new Error('TCO_REGISTRO_EXISTE', 1));
            }
        }catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }

    private function notExistsGramatura($row){
        $register = $this->service->getGramatura($row->getArrayCopy());
        return !(bool)$register;
    }
    
    //apaga regras de Gramatura do cardápio base
    public function deletaRegraGramatura(DTO\Request $request, DTO\Response $response) {
        $this->entityManager->beginTransaction();
        try{
            $rowToDelete = $request->getRow();
            $nrgramaturacardbase = $rowToDelete['NRGRAMATURACARDBASE'];
            $usoEmPropostas = $this->service->verificaUsoPropostaComercial($nrgramaturacardbase);
            if(empty($usoEmPropostas) || (count($usoEmPropostas) == 0)){
                $this->service->deletaAgrupamentoRegraGramatura($nrgramaturacardbase);
                $rows = $this->crudService->getRowsToDelete($request, "gramaturacardbase");
                $dataSet = new DataSet("gramaturacardbase", $rows);
        		$this->crudService->delete($response, $dataSet);
        		
                $this->entityManager->commit();
            }
            else{
                $response->addDataSet(new DataSet("erros", array('uso_em_propostas' => $usoEmPropostas)));
                $this->entityManager->rollback();
            }
        }catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    //retorna os valores referentes aos campos dinamicos para o grid de gramatura por produto da regra de Gramatura
    public function retornaCamposDinamicosGramatura(DTO\Request $request, DTO\Response $response) {
        $filtro = $request->getRow();
        $inString = '';
        
        $conditions = [
            array(
                "columnName" => "NRORG",
                "operator"   => '=',
                "value"      => $this->environment->getNrOrgTrab()
            )
        ];
        $colunasDinamicas = $this->service->retornaColsPorcionamento($filtro['CDGRTIPOPR']);
        $colunasDinamicas = new DataSet('porcionamento', $colunasDinamicas);
        $response->addDataSet($colunasDinamicas);
        
        foreach ($colunasDinamicas->getRows() as $value) {
            $inString .= "'".$value['NRPORCIONAMENTO']."', ";
        }
        $inString = substr(trim($inString), 0, -1);
        
        $campos = $this->service->retornaCamposDinamicosGramatura($inString, $filtro['NRGRAMATURACARDBASE']);
        $dataSet = new DataSet("campos", $campos);
        $response->addDataSet($dataSet);
    }
    
    //salva os agrupamentos referentes a uma regra de gramatura do cardápio base
    public function salvaGramaturaAgrupamento(DTO\Request $request, DTO\Response $response) {
        $regra = $request->getRow();
        $this->entityManager->beginTransaction();
        
        $this->service->deletaAgrupamentoRegraGramatura($regra['NRGRAMATURACARDBASE']);
        try{
            if(!empty($regra['AGRUPAMENTO'])){
                
                foreach ($regra['AGRUPAMENTO'] as $key => $value) {
                    $regra['AGRUPAMENTO'][$key]['NRITEMGRAMCARDBASE'] = $this->newCode->getCodeByTableName('ITEMGRAMCARDBASE', $this->environment->getNrOrgTrab());
                }
                
                $dataSet = new DataSet("toSave", $regra['AGRUPAMENTO']);
                $request = new DTO\Request\DataSet($dataSet, '', '', '');
                $rows    = $this->crudService->getRowsToSave($request, "itemgramcardbase");
                $this->crudService->save("itemgramcardbase", $rows);
            }
            $this->entityManager->commit();
        }catch(\Exception $e){
            throw $e;
            $this->entityManager->rollback();
        }
    }
    
    //salva as regras de particularidade por segmento do cardápio base
    public function salvaRegraParticularidade(DTO\Request $request, DTO\Response $response) {
        $regra = $request->getRow();
        $this->entityManager->beginTransaction();
        try{
            $existe = $this->service->existeParticularidadeSemelhante($regra);
            if(!$existe){
                if($regra['__is_new']){
                    $regra['NRPARTSEGCARDBASE'] = $this->newCode->getCodeByTableName('PARTSEGCARDBASE', $this->environment->getNrOrgTrab());
                    $regra['CDPARTSEGCARDBASE'] = $this->service->retornaProxCodPartic($regra['NRCARDAPIOBASE']);
                }
                $rows = $this->crudService->getRowsToSave($request, "partsegcardbase");
                $this->crudService->save("partsegcardbase", $rows);
    
                $this->entityManager->commit();
                $response->addDataSet(new DataSet("chaves", array('NRPARTSEGCARDBASE' => $regra['NRPARTSEGCARDBASE'], 'CDPARTSEGCARDBASE'=> $regra['CDPARTSEGCARDBASE'])));
            }else{
                $response->addDataSet(new DataSet("erro", array()));
            }
        }catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    //apaga as regras de particularidade por segmento do cardápio base
    public function deletaRegraParticularidade(DTO\Request $request, DTO\Response $response) {
        $this->entityManager->beginTransaction();
        try{
            $rowToDelete = $request->getRow();
            $this->service->deletaTipoPratoRegraParticularidade($rowToDelete['NRPARTSEGCARDBASE']);
            
            $rows = $this->crudService->getRowsToDelete($request, "partsegcardbase");
            $dataSet = new DataSet("partsegcardbase", $rows);
    		$this->crudService->delete($response, $dataSet);
    		
            $this->entityManager->commit();
        }catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function salvaTipoPratoPart(DTO\Request $request, DTO\Response $response) {
        $rowsToSave = $request->getDataSet()->getRows();
        $dataSet = array();
        foreach ($rowsToSave as $row) {
            if(!$row['NRITPARTSEGCARDBASE']){
                $row['__is_new'] = true;
                $row['NRITPARTSEGCARDBASE'] = $this->newCode->getCodeByTableName('ITPARTSEGCARDBASE', $this->environment->getNrOrgTrab());
            }
            array_push($dataSet, $row);
        }
        
        $dataSet = new DataSet("toSave", $dataSet);
        $request = new DTO\Request\DataSet($dataSet, '', '', '');
        $rows    = $this->crudService->getRowsToSave($request, "itpartsegcardbase");
        $this->crudService->save("itpartsegcardbase", $rows);
    }

    /**
     * Preenche os dados da aba de Descartáveis.
     * Busca os campos de Amostra Quente e Frio e
     * carrega o dataSource do grid inferior da tela.
     * 
     * @param {DTO\Request} $request Requisição vinda do
     *  cliente.
     * @param {DTO\Response} $response Resposta a requisição
     *  do usuário.
     */
    public function getDescartaveisData(DTO\Request $request, DTO\Response $response){
        $nrCardapioBase = $request->getRow()->get('NRCARDAPIOBASE')['value'];

        $dataSetToResponse = array(
            'AMOSTRA'=>$this->service->getAmostraData($nrCardapioBase),
            'DESCART'=>$this->service->getDescartaveisCardBase($nrCardapioBase)
        );

        $response->addDataSet(new DataSet('descartData', $dataSetToResponse));
    }

    /**
     * Salva inserção ou alteração nas amostras
     * de descartáveis do cardapio base.
     * 
     * @param {DTO\Request} $request Requisição vinda do
     *  cliente.
     * @param {DTO\Response} $response Resposta a requisição
     *  do usuário.
     */
    public function saveAmostra(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow()->getArrayCopy();
        
        $isValidAmostraToSave = $this->validateAmostraToSave($row);
        
        if($isValidAmostraToSave=='VALID'){
            $this->service->saveAmostra($row);
        } else {
            $response->setError(new Error($isValidAmostraToSave, 1));
        }
    }

    /**
     * Verifica se tipos de prato de amostra de 
     * descartaveis são válidos para salvar.
     * 
     * @param {Array} $row Registro contendo
     *  dados de configcardbase para verificação.
     * 
     * @return {String} 'VALID' caso seja valido
     *  ou código da mensagem de erro caso invalido.
     */
    private function validateAmostraToSave($row){
        $isValidAmostraFromCardBase = $this->service->isValidAmostraFromCardBase($row);
        $result = 'VALID';

        if(!$isValidAmostraFromCardBase){
            $result = 'TCO01006_INVALID_AMOSTRA_CONFCARDBASE';
        }

        return $result;
    }

    /**
     * Altera forma de calculo dos registros 
     * passados por parametros.
     */
    public function changeFormaCalculo(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow()->getArrayCopy();
        $this->service->changeFormaCalculo($row);
    }

    /**
     * Salva alteração nas quantidades iniciais
     * e finais de tipos de prato de limpeza.
     * 
     * @param {DTO\Request} $request Requisição vinda do
     *  cliente.
     * @param {DTO\Response} $response Resposta a requisição
     *  do usuário.
     */
    public function saveLimpeza(DTO\Request $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        
        $isValidOperation = $this->service->saveLimpeza($rows);
        if($isValidOperation!='VALID'){
            $response->setError(new Error($isValidOperation, 1));
        }
    }


    /**
     * Remove registros de limpeza do cardapio
     * base.
     * 
     * @param {DTO\Request} $request Requisição vinda do
     *  cliente.
     * @param {DTO\Response} $response Resposta a requisição
     *  do usuário.
     */
    public function removeLimpezaCardBase(DTO\Request $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        $rows = array_map(function($reg){
            return $reg->get('NRLIMPEZACARDBASE');
        }, $rows);
        
        $this->service->removeLimpezaCardBase($rows);
    }
    
    /**
     * Duplica uma regra de adesão
     * do cardapio base.
     * 
     * @param {DTO\Request} $request Requisição vinda do
     *  cliente.
     * @param {DTO\Response} $response Resposta a requisição
     *  do usuário.
     */
    public function duplicaRegraAdesao(DTO\Request $request, DTO\Response $response){
        $filtro = $request->getRow();
        
        $conditions = [
            array(
                "columnName" => "NRADESAOCARDBASE",
                "operator"   => '=',
                "value"      => $filtro['NRADESAOCARDBASE']
            )
        ];
        try{
            $this->entityManager->beginTransaction();
            $regraOriginal = $this->crudService->getFilteredDataSet('adesaocardbase', $conditions);
            $regraOriginal = $regraOriginal->getRows()[0];
            $regraCopia    = array();
            
            $regraCopia["NRADESAOCARDBASE"] = $this->newCode->getCodeByTableName('ADESAOCARDBASE', $this->environment->getNrOrgTrab());
            $regraCopia["CDADESAOCARDBASE"] = $this->service->retornaProxCodAdesaoCard($regraOriginal['NRCARDAPIOBASE']);
            $regraCopia["NRCARDAPIOBASE"]   = $regraOriginal["NRCARDAPIOBASE"];
            $regraCopia["NMADESAOCARDBASE"] = strlen($regraOriginal["NMADESAOCARDBASE"].' (Cópia)') < 50 ? $regraOriginal["NMADESAOCARDBASE"].' (Cópia)' : $regraOriginal["NMADESAOCARDBASE"];
            $regraCopia["CDGRTIPOPR"]       = $regraOriginal["CDGRTIPOPR"];
            $regraCopia["NRDIACARDBASE"]    = $regraOriginal["NRDIACARDBASE"];
            $regraCopia["IDATIVO"]          = $regraOriginal["IDATIVO"];
            $regraCopia["__is_new"]         = true;
            $regraCopia['CDSEGMER']         = $this->service->retornaRamosAdesao($filtro['NRADESAOCARDBASE']);
            $regraCopia['NRATIVCLISEGMER']  = $this->service->retornaSegmentosAdesao($filtro['NRADESAOCARDBASE']);
            
            $dataSet = new DataSet("toSave", array($regraCopia));
            $request = new DTO\Request\DataSet($dataSet, '', '', '');
            $rows    = $this->crudService->getRowsToSave($request, "adesaocardbase");
            $this->crudService->save("adesaocardbase", $rows);
            
            $regraCopia['TIPOSPRATO'] = $this->service->retornaTiposPratoAdesao($regraOriginal['NRADESAOCARDBASE']);
            $this->salvaTiposDePrato($regraCopia);
            $this->salvaSegmentoAdesao($regraCopia);
            $this->salvaRamosAdesao($regraCopia);
            $this->entityManager->commit();
            $response->addDataSet(new DataSet("chaves", array('NRADESAOCARDBASE' => $regraCopia['NRADESAOCARDBASE'], 'CDADESAOCARDBASE'=> $regraCopia['CDADESAOCARDBASE'])));
        }catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    /**
     * Rotina de importação de itens de gramatura.
     * 
     * @param {DTO\Request} $request Requisição vinda do
     *  cliente.
     * @param {DTO\Response} $response Resposta a requisição
     *  do usuário.
     */
    public function importaItemGramatura(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $itensOrigem = $this->service->returnItensGramatura($row['NRGRAMATURACARDBASE']);
        $itensDestino = $this->service->returnItensGramatura($row['NRGRAMATURACARDBASEDEST']);
        if (sizeof($itensOrigem) > 0) {
            $this->excluiItemCardbase($itensDestino, $response);
            $rowsToSave = $this->buildRowsToSave($itensOrigem, $row);
            $this->salvaItemCardbase($rowsToSave, $response);
        } else {
            $response->addDataSet(new DataSet('errormessages', array(
                'message' => 'TCO01006_GRAMATURADEST_SEM_ITENS'
            )));
        }
    }
    
    private function excluiItemCardbase($rows, $response) {
        $dataSet = new DataSet("itemgramcardbase", $rows);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rows = $this->crudService->getRowsToDelete($request, "itemgramcardbase");
        $dataSet = new DataSet("itemgramcardbase", $rows);
		$this->crudService->delete($response, $dataSet);
    }
    
    private function buildRowsToSave($itens, $row) {
        foreach($itens as &$item) {
            $item["NRITEMGRAMCARDBASE"] = $this->newCode->getCodeByTableName('ITEMGRAMCARDBASE', $this->environment->getNrOrgTrab(), 1, 10, true);
            $item["NRGRAMATURACARDBASE"] = $row['NRGRAMATURACARDBASEDEST'];
            $item["__is_new"] = true;
        }
        return $itens;
    }
    
    private function salvaItemCardbase($rows, $response) {
        $dataSet = new DataSet("itemgramcardbase", $rows);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rows = $this->crudService->getRowsToSave($request, "itemgramcardbase");
		$this->crudService->save("itemgramcardbase", $rows);
        $response->addDataSet(new DataSet("itemgramcardbase", $rows));
    }
}
