<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;

class Tco01400CustoPelaMedia extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService; 
    protected $parameterBag; 
    protected $environment; 
    protected $entityManager; 
    
    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        $this->crudService   = $crudService; 
        $this->parameterBag  = $parameterBag; 
        $this->environment   = $environment; 
        $this->entityManager = $entityManager; 
    }
    
    public function adicionaNovoProduto(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $row["__is_new"] = true;
        $row["__is_new_code"] = true;
        $row["__new_code_pk"] = "PRODCUSMEDIATCO";
        $row["__table_pk"] = "NRPRODCUSMEDIATCO";
        $row["__new_code_length"] = 1;
        
        $produtoEncontrado = $this->produtoSelecionadoExistente($row);
        
        if(!$produtoEncontrado) {
            $this->salvaNovoProduto(array($row));
            $response->addDataSet(new DataSet("prodcusmediatco", array()));
        } else {
            $validacoes = array(
                "message" => "TCO01400_PRODUTO_EM_USO",
                "params" => $produtoEncontrado
            );
            $response->addDataSet(new DataSet("validacoes", $validacoes));
        }
    }
    
    private function produtoSelecionadoExistente($row) {
        
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT PRODCUSMEDIATCO.NRPRODCUSMEDIATCO, 
                       PRODCUSMEDIATCO.CDPRODUTO,
                       PRODUTO.NMPRODUTO,
                       PRODCUSMEDIATCO.NRORG
                  FROM PRODCUSMEDIATCO
                 INNER JOIN PRODUTO
                    ON PRODUTO.CDPRODUTO = PRODCUSMEDIATCO.CDPRODUTO
                 WHERE PRODUTO.CDARVPROD = :CDARVPROD
                       /* (INSTR(:CDARVPROD, PRODUTO.CDARVPROD) = 1 OR
                       INSTR(PRODUTO.CDARVPROD, :CDARVPROD) = 1) */
                   AND PRODCUSMEDIATCO.NRORG = :NRORG
                 ORDER BY PRODUTO.CDARVPROD ASC";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "CDARVPROD"  => $row["CDARVPROD"],
            "NRORG"      => $row["NRORG"]
        ); 
        
        $query->execute($params);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        
        return $result;
        
    }
    
    private function salvaNovoProduto($row) {
        $newdataSet = new DataSet("prodcusmediatco", $row);
        $newrequest = new DTO\Request\DataSet($newdataSet, "", "", "");
        $rows = $this->crudService->getRowsToSave($newrequest, "prodcusmediatco");
        $this->crudService->save("prodcusmediatco", $rows);
    }

    public function excluiProdutosSelecionados(DTO\Request $request, DTO\Response $response) {
        $this->excluiProdutos($request, $response);
    }
    
    private function excluiProdutos($request, $response) {
        $rows = $this->crudService->getRowsToDelete($request, 'prodcusmediatco');
        $dataSet = new DataSet("prodcusmediatco", $rows);
		$this->crudService->delete($response, $dataSet);
    }
    
}